// ---------------------------------------------------------------------------
//
// iRoboter.PAS                                          Ulrich Mller 23.1.00
//                                                       Procida 400NT Delphi4
// fischertechnik Industry Robots : Knickarm Roboter
// Robotersteuerung
//
// ---------------------------------------------------------------------------

unit iRoboter;

interface

uses Windows, umFishEx;

type
  statustype_ = (auf, zu);

var
  PortName: string;
  ist:        array[1..3] of Integer; // --- Ist-Positionen
  gzustand:   string[5];              // --- Greiferzustand
  gkonstante: Integer;                // --- Greifzangenkonstante
  gfaktor:    Byte;                   // --- Greifzangenfaktor
  nachlauf:   Byte;                   // --- Nachlaufkorrektur
  nl_zeit:    Integer;                // --- Nachlaufzeit
  segmente:   Integer;                // --- Anzahl Segmente pro Eingabe
  check:      Boolean;                // --- Arbeitsraumberwachung

procedure Robot(soll1_, soll2_, soll3_: Integer);
procedure Greifer(zustand_: statustype_);
procedure Home;
function  Abbruch:Boolean;

implementation
const
  EMaske: array[1..8] of LongInt= ($1, $2, $4, $8, $10, $20, $40, $80);

function Abbruch:Boolean;
// --- Abbruch Robot Operation durch Excape-Taste ---------------
begin
  if(GetAsyncKeyState(VK_ESCAPE) <> 0) then Result := true else Result := false;
end;

procedure Robot(soll1_, soll2_, soll3_: Integer);
// --- Fahren des Robots simultan mit bis zu drei Motoren -----
var
  i_, j_:           Byte;
  f_, o_:           array[1..3] of Boolean;
  p_, diff_, soll_: array[1..3] of Integer;

procedure ArbeitsraumUeberwachen;
begin       // --- bei Robot85 komplexer, da Armstellungen sich beeinflussen
  if soll1_ > 240 then soll1_ := 240;
  if soll1_ < 0    then soll1_ := 0;
  if soll2_ > 160 then soll2_ := 160;
  if soll2_ < 0 then soll2_ := 0;
  if soll3_ >112 then soll3_ := 112;
  if soll3_ < 0 then soll3_ := 0;
end;

procedure Input_;
// --- Abfrage der Impulstaster ---
var i_: Cardinal; iStatus: LongInt;
begin
  iStatus := ftiGetInputs;    // --- um wiederholten Interfacezugriff zu vermeiden
  for i_ := 1 to 3 do begin
    if (iStatus AND EMaske[i_*2] = 0) then f_[i_] := false else f_[i_] := true;
    if o_[i_] <> f_[i_] then Inc(p_[i_]);
    o_[i_] := f_[i_];
    end;
end;

begin
  if check then ArbeitsraumUeberwachen;
  soll_[1] := soll1_;
  soll_[2] := soll2_;
  soll_[3] := soll3_;
  for i_ := 1 to 3 do begin
    f_[i_]    := false;
    o_[i_]    := false;
    p_[i_]    := 0;
    diff_[i_] := 0;
    if soll_[i_] < ist[i_] then ftiSetMotor(i_, ftiLinks) // Robot85 : vertauscht
    else if soll_[i_] > ist[i_] then ftiSetMotor(i_, ftiRechts);
    end;
  repeat
    Input_;
    for i_ := 1 to 3 do begin
      if soll_[i_] > ist[i_] then diff_[i_] := soll_[i_] - (ist[i_] + p_[i_]);
      if soll_[i_] < ist[i_] then diff_[i_] := ist[i_] - (soll_[i_] + p_[i_]);
      if (diff_[i_] = 0) then ftiSetMotor(i_, ftiAus);
      end;
  until (diff_[1] <= 0) AND (diff_[2] <= 0) AND (diff_[3] <= 0) OR (Abbruch);
  for j_ := 1 to nl_zeit do Input_;
  for i_ := 1 to 3 do
    if soll_[i_] > ist[i_] then Inc(ist[i_], p_[i_])
    else if soll_[i_] < ist[i_] then Dec(ist[i_], p_[i_]);
end;

procedure Greifer(zustand_: statustype_);
// --- Greifer ffnen/schlieen ---------
begin
  case zustand_ of
  zu: begin
    if ftiGetInput(7) then begin
      ftiSetMotor(4, ftiRechts);
      ftiWaitForChange(8, gfaktor, 0);
      ftiSetMotor(4, ftiAus);
      gzustand := '   zu';
      end;
    end;
  auf: begin
    ftiSetMotor(4, ftiLinks);
    ftiWaitForInput(7, true);
    ftiSetMotor(4, ftiAus);
    gzustand := '  auf';
    end;
  end;
end;

procedure Home;
// --- Anfahren der HomePosition, simultan --------------------

var i_, h1_, h2_, h3_, h4_: Integer;
begin
  h1_ := 1; h2_ := 1; h3_ := 1; h4_ := 1;
  repeat
    if ftiGetInput(1) AND (h1_ = 1) then ftiSetMotor(1, ftiRechts);
    if ftiGetInput(3) AND (h2_ = 1) then ftiSetMotor(2, ftiRechts);
    if ftiGetInput(5) AND (h3_ = 1) then ftiSetMotor(3, ftiRechts);
    if NOT ftiGetInput(7) AND (h4_ = 1) then ftiSetMotor(4, ftiLinks);
    if NOT ftiGetInput(1) then begin ftiSetMotor(1, ftiLinks); h1_ := -1; end;
    if NOT ftiGetInput(3) then begin ftiSetMotor(2, ftiLinks); h2_ := -1; end;
    if NOT ftiGetInput(5) then begin ftiSetMotor(3, ftiLinks); h3_ := -1; end;
    if ftiGetInput(7) then begin ftiSetMotor(4, ftiAus); h4_ := 0; end;
    if ftiGetInput(1) AND (h1_ = -1) then begin ftiSetMotor(1, ftiAus); h1_ := 0; end;
    if ftiGetInput(3) AND (h2_ = -1) then begin ftiSetMotor(2, ftiAus); h2_ := 0; end;
    if ftiGetInput(5) AND (h3_ = -1) then begin ftiSetMotor(3, ftiAus); h3_ := 0; end;
  until (h1_ = 0) AND (h2_ = 0) AND (h3_ = 0) AND (h4_ = 0) OR Abbruch;
  gzustand := '  auf';
  for i_ := 1 to 3 do ist[i_] := 0;
end;

initialization
  nachlauf   := 1;     // --- Nachlaufkorrektur           (Robot85 : 4)
  nl_zeit    := 60;    // --- Nachlaufzeit                         (500)
  segmente   := 8;     // --- Anzahl Segmente pro Eingabe          (64)
  gkonstante := 1000;  // --- Greifzangenkonstante, nicht benutzt  (1000)
  gfaktor    := 24;    // --- Greifzangenfaktor                    (5)
  check      := true;  // --- Arbeitsraumberwachung               (true)
end.

